import configparser
import re
from collections import deque


class ConfigException(Exception):
    pass


def load_from_ini_file(path, default_modules=()):
    ini_config = configparser.ConfigParser()
    ini_config.read(path)

    config = Config()
    for config_name in ini_config.sections():
        vhost = VirtualHost(dict(ini_config[config_name]))
        config.add(config_name, vhost)

    for module_class in default_modules:
        config.modules.append(module_class())

    return config


class VirtualHost(dict):
    def matches(self, request):
        location = self['location']
        if request.uri == location:
            return True

        return re.compile(location).match(request.uri) is not None


class Config(dict):
    def __init__(self, *args, **kwargs):
        super(dict, self).__init__(*args, **kwargs)
        self.virtual_hosts = []
        self.modules = deque()

    def add(self, name, virtual_host):
        if name == 'server':
            self.update(virtual_host)
            return

        self.virtual_hosts.append(virtual_host)

    def process(self):
        for module in self.modules:
            module.process_config(self)

        for vhost in self.virtual_hosts:
            for module in self.modules:
                module.process_config(vhost)

    def match(self, request):
        current_config = self

        for virtual_host in self.virtual_hosts:
            if virtual_host.matches(request):
                current_config = virtual_host
                break

        return current_config
