import logging


def get_server_logger(level_=logging.INFO):
    formatter = logging.Formatter('[%(levelname)s] %(asctime)s: %(message)s')

    handler = logging.StreamHandler()
    handler.setFormatter(formatter)

    logger = logging.getLogger('tak_server')
    logger.addHandler(handler)
    logger.setLevel(level_)

    return logger


def get_http_logger(format_, level_, access_log):
    logger = logging.getLogger('http_handler')
    logger.setLevel(level_)

    formatter = logging.Formatter(format_, style='{')
    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(formatter)
    logger.addHandler(stream_handler)

    if access_log is not None:
        file_handler = logging.FileHandler(filename=access_log)
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)

    return logger


class LogModule:
    def process_config(self, config):
        if 'log_format' not in config:
            config['logger_format'] = '{ip} [{asctime}] "{request.method} {request.uri} {request.http_version}" '\
                                      '{response.status_code} {response_length} "{request.referer}" '\
                                      '"{request.user-agent}"'

        access_log = None
        if 'access_log' in config:
            access_log = config['access_log']

        config['logger_level'] = logging.DEBUG
        config['logger_http'] = get_http_logger(config['logger_format'], config['logger_level'], access_log)

