import pkg_resources
import pystache

from tak.http import Response, HTTP_OK


class ViewModule:
    def __init__(self):
        self._view = None

    @property
    def view(self):
        if not self._view:
            templates_path = pkg_resources.resource_filename('tak', 'templates/')
            partials_path = pkg_resources.resource_filename('tak', 'templates/partials/')

            self._view = pystache.Renderer(search_dirs=[templates_path, partials_path])
        return self._view

    def render(self, template_name, context):
        return self.view.render_name(template_name, context)

    def render_response(self, template_name, context):
        content = self.render(template_name, context)

        response = Response(HTTP_OK, content)
        response.add_header('Content-Type', 'text/html')
        return response

    def process_config(self, config):
        config['view'] = ViewModule
